@@ -4,10 +4,11 @@ Created on Wed May 30 12:05:46 2018
 
 @author: MichaelEK
 """
+import numpy as np
 from hilltoppy.web_service import measurement_list, site_list, get_data, wq_sample_parameter_list
 
 ### Parameters
-base_url = 'http://wateruse.ecan.govt.nz'
+base_url = 'http://data.ecan.govt.nz/'
 hts = 'WQAll.hts'
 site = 'SQ31045'
 measurement = 'Total Phosphorus'
@@ -19,7 +20,7 @@ dtl_method = 'trend'
 
 
 def test_site_list():
-    sites = site_list(base_url, hts)
+    sites = site_list(base_url, hts, True)
     assert len(sites) > 9000
 
 
@@ -50,5 +51,4 @@ def test_get_data3():
 
 def test_get_data4():
     tsdata4, extra4 = get_data(base_url, hts, site, measurement, from_date=from_date, to_date=to_date, parameters=True, dtl_method=dtl_method)
-    assert (len(tsdata4) > 80) & (len(extra4) > 300) & (tsdata4.Value.dtype.name == 'float32')
-
+    assert (len(tsdata4) > 80) & (len(extra4) > 300) & (tsdata4.Value.dtype == np.number)
