@@ -40,7 +40,7 @@ gauging_dict = {'Stage': {'row': 'I1', 'multiplier': 0.001},
 ### Functions
 
 
-def build_url(base_url, hts, request, site=None, measurement=None, from_date=None, to_date=None, location=False, site_parameters=None, agg_method=None, agg_interval=None, alignment=None):
+def build_url(base_url, hts, request, site=None, measurement=None, from_date=None, to_date=None, location=None, site_parameters=None, agg_method=None, agg_interval=None, alignment=None, quality_codes=False, tstype=None):
     """
     Function to generate the Hilltop url for the web service.
 
@@ -60,14 +60,20 @@ def build_url(base_url, hts, request, site=None, measurement=None, from_date=Non
         The start date in the format 2001-01-01. None will put it to the beginning of the time series.
     to_date : str or None
         The end date in the format 2001-01-01. None will put it to the end of the time series.
-    location : bool
-        Should the location be returned? Only applies to the SiteList request.
+    location : str or bool
+        Should the location be returned? Only applies to the SiteList request. 'Yes' returns the Easting and Northing, while 'LatLong' returns NZGD2000 lat lon coordinates.
     site_parameters : list
         A list of the site parameters to be returned with the SiteList request.
     agg_method : str
         The aggregation method to resample the data. e.g. Average, Total, Moving Average, Extrema.
     agg_interval : str
         The aggregation interval for the agg_method. e.g. '1 day', '1 week', '1 month'.
+    alignment : str
+        The time alignment in the form '00:00'.
+    quality_codes : bool
+        Should the quality codes get returned from the GetData function.
+    tstype : str
+        The timeseries type, one of Standard, Check or Quality
 
     Returns
     -------
@@ -96,8 +102,22 @@ def build_url(base_url, hts, request, site=None, measurement=None, from_date=Non
 #        url = url + '&Collection=' + requests.utils.quote(collection)
     if isinstance(site_parameters, list):
         url = url + '&SiteParameters=' + requests.utils.quote(','.join(site_parameters))
-    if location and (request == 'SiteList'):
-        url = url + '&Location=Yes'
+    if isinstance(location, (str, bool)) and (request == 'SiteList'):
+        if isinstance(location, bool):
+            if location:
+                url = url + '&Location=Yes'
+        else:
+            url = url + '&Location=' + location
+    if quality_codes and (request == 'GetData'):
+        url = url + '&ShowQuality=Yes'
+
+    if tstype and (request == 'GetData'):
+        if tstype == 'Standard':
+            url = url + '&tsType=StdSeries'
+        elif tstype == 'Quality':
+            url = url + '&tsType=StdQualSeries'
+        elif tstype == 'Check':
+            url = url + '&tsType=CheckSeries'
 
     ### Time interval goes last!
     if request == 'GetData':
@@ -120,7 +140,7 @@ def build_url(base_url, hts, request, site=None, measurement=None, from_date=Non
     return url
 
 
-def site_list(base_url, hts, location=False):
+def site_list(base_url, hts, location=None, measurement=None):
     """
     SiteList request function. Returns a list of sites associated with the hts file.
 
@@ -130,28 +150,35 @@ def site_list(base_url, hts, location=False):
         root url str
     hts : str
         hts file name including the .hts extension.
-    location : bool
-        Should the Easting and Northing be returned?
+    location : str or bool
+        Should the location be returned? Only applies to the SiteList request. 'Yes' returns the Easting and Northing, while 'LatLong' returns NZGD2000 lat lon coordinates.
 
     Returns
     -------
     DataFrame
     """
-    url = build_url(base_url, hts, 'SiteList', location=location)
-    resp = requests.get(url)
+    url = build_url(base_url, hts, 'SiteList', location=location, measurement=measurement)
+    resp = requests.get(url, timeout=300)
     tree1 = ET.fromstring(resp.content)
     site_tree = tree1.findall('Site')
-    if location:
+    if isinstance(location, (str, bool)):
         sites_dict = {}
         for s in site_tree:
             site1 = s.attrib['Name']
-            children = s.getchildren()
+            # children = s.getchildren()
+            children = list(s)
             if len(children) == 2:
-                locs1 = [int(l.text) for l in children]
+                locs1 = [float(l.text) for l in children]
                 sites_dict.update({site1: locs1})
             else:
                 sites_dict.update({site1: [np.nan, np.nan]})
-        sites_df = pd.DataFrame.from_dict(sites_dict, orient='index', columns=['Easting', 'Northing'])
+
+        if (location == 'Yes') or (location == True):
+            cols = ['Easting', 'Northing']
+        elif location == 'LatLong':
+            cols = ['lat', 'lon']
+
+        sites_df = pd.DataFrame.from_dict(sites_dict, orient='index', columns=cols)
         sites_df.index.name = 'SiteName'
         sites_df.reset_index(inplace=True)
     else:
@@ -160,7 +187,7 @@ def site_list(base_url, hts, location=False):
     return sites_df
 
 
-def measurement_list(base_url, hts, site, measurement=None, output_bad_sites=False):
+def measurement_list(base_url, hts, site, measurement=None, output_bad_sites=False, tstype="Standard"):
     """
     Function to query a Hilltop server for the measurement summary of a site.
 
@@ -176,6 +203,8 @@ def measurement_list(base_url, hts, site, measurement=None, output_bad_sites=Fal
         The measurement type name.
     output_bad_sites : bool
         Should sites with problems be returned?
+    tstype : str
+        Which type of dataseries are to be output, Standard or All?
 
     Returns
     -------
@@ -183,14 +212,19 @@ def measurement_list(base_url, hts, site, measurement=None, output_bad_sites=Fal
         indexed by Site and Measurement.
         If output_bad_sites is True than two DataFrames are returned.
     """
-    ds_types = ['DataType', 'From', 'To', 'SensorGroup']
+    # Add the TSType field to the output dataframe if all measurements are requested.
+    if tstype != "All":
+        ds_types = ['DataType', 'From', 'To', 'SensorGroup']
+    else:
+        ds_types = ['DataType', 'From', 'To', 'SensorGroup', 'TSType']
+
     m_types = ['RequestAs', 'Units']
 
     ### Make url
     url = build_url(base_url, hts, 'MeasurementList', site, measurement)
 
     ### Request data and load in xml
-    resp = requests.get(url)
+    resp = requests.get(url, timeout=300)
     tree1 = ET.fromstring(resp.content)
     if tree1.find('Error') is not None:
         raise ValueError('No results returned from URL request')
@@ -215,7 +249,7 @@ def measurement_list(base_url, hts, site, measurement=None, output_bad_sites=Fal
     for d in data_sources:
         if d.find('TSType') is None:
             pass
-        elif d.find('TSType').text != 'StdSeries':
+        elif d.find('TSType').text != 'StdSeries' and tstype != "All":
             continue
         ds_dict = {c.tag: c.text.encode('ascii', 'ignore').decode() for c in d if c.tag in ds_types}
         m_all = d.findall('Measurement')
@@ -274,7 +308,7 @@ def measurement_list_all(base_url, hts):
     sites = site_list(base_url, hts)
 
     mtype_list = []
-    for s in sites:
+    for s in sites.SiteName:
 #        print(s)
         m1 = measurement_list(base_url, hts, s)
         mtype_list.append(m1)
@@ -283,7 +317,7 @@ def measurement_list_all(base_url, hts):
     return mtype_df
 
 
-def get_data(base_url, hts, site, measurement, from_date=None, to_date=None, agg_method=None, agg_interval=None, alignment='00:00', parameters=False, dtl_method=None):
+def get_data(base_url, hts, site, measurement, from_date=None, to_date=None, agg_method=None, agg_interval=None, alignment='00:00', parameters=False, dtl_method=None, quality_codes=False, tstype=None, ignore_gaps=True):
     """
     Function to query a Hilltop web server for time series data associated with a Site and Measurement.
 
@@ -311,6 +345,12 @@ def get_data(base_url, hts, site, measurement, from_date=None, to_date=None, agg
         Should the additional parameters (other than the value) be extracted and returned if they exist?
     dtl_method : str or None
         The method to use to convert values below a detection limit to numeric. Used for water quality results. Options are 'half' or 'trend'. 'half' simply halves the detection limit value, while 'trend' uses half the highest detection limit across the results when more than 40% of the values are below the detection limit. Otherwise it uses half the detection limit.
+    quality_codes : bool
+        Should the quality codes get returned?
+    tstype : str
+        The timeseries type, one of Standard, Check or Quality
+    ignore_gaps : bool
+        Should gaps be ignored when pulling out the data? Different organisations handle gap tagging differently. Some put gap tags at the beginning and end of a gap, while others put only a single gap tag. If ignore_gaps is False, then hilltop-py assumes the former and removes data between gaps. When in doubt, use the default of True.
 
     Returns
     -------
@@ -318,10 +358,10 @@ def get_data(base_url, hts, site, measurement, from_date=None, to_date=None, agg
         If parameters is False, then only one DataFrame is returned indexed by Site, Measurement, and DateTime. If parameters is True, then two DataFrames are returned. The first is the same as if parameters is False, but the second contains those additional parameters indexed by Site, Measurement, DateTime, and Parameter.
     """
     ### Make url
-    url = build_url(base_url=base_url, hts=hts, request='GetData', site=site, measurement=measurement, from_date=from_date, to_date=to_date, agg_method=agg_method, agg_interval=agg_interval, alignment=alignment)
+    url = build_url(base_url=base_url, hts=hts, request='GetData', site=site, measurement=measurement, from_date=from_date, to_date=to_date, agg_method=agg_method, agg_interval=agg_interval, alignment=alignment, quality_codes=quality_codes, tstype=tstype)
 
     ### Request data and load in xml
-    resp = requests.get(url)
+    resp = requests.get(url, timeout=300)
     tree1 = ET.fromstring(resp.content)
     if tree1.find('Error') is not None:
         raise ValueError('No results returned from URL request')
@@ -369,8 +409,9 @@ def get_data(base_url, hts, site, measurement, from_date=None, to_date=None, agg
                 if tag1 == 'Gap':
                     gap = gap + 1
                     continue
-                if gap % 2 == 1:
-                    continue
+                if not ignore_gaps:
+                    if gap % 2 == 1:
+                        continue
                 tsdata_list.append([d.find('T').text, d.find('Value').text.encode('ascii', 'ignore').decode()])
             data_df = pd.DataFrame(tsdata_list, columns=['DateTime', 'Value'])
     elif datatype in ['SimpleTimeSeries', 'MeterReading']:
@@ -381,10 +422,19 @@ def get_data(base_url, hts, site, measurement, from_date=None, to_date=None, agg
             if tag1 == 'Gap':
                 gap = gap + 1
                 continue
-            if gap % 2 == 1:
-                continue
-            tsdata_list.append([d.find('T').text, d.find('I1').text.encode('ascii', 'ignore').decode()])
-        data_df = pd.DataFrame(tsdata_list, columns=['DateTime', 'Value'])
+            if not ignore_gaps:
+                if gap % 2 == 1:
+                    continue
+
+            if quality_codes:
+                tsdata_list.append([d.find('T').text, d.find('I1').text.encode('ascii', 'ignore').decode(), d.find('Q1').text.encode('ascii', 'ignore').decode()])
+            else:
+                tsdata_list.append([d.find('T').text, d.find('I1').text.encode('ascii', 'ignore').decode()])
+
+        if quality_codes:
+            data_df = pd.DataFrame(tsdata_list, columns=['DateTime', 'Value', 'QualityCode'])
+        else:
+            data_df = pd.DataFrame(tsdata_list, columns=['DateTime', 'Value'])
         data_df.Value = pd.to_numeric(data_df.Value, errors='ignore')
 
     elif datatype in ['GaugingResults']:
@@ -398,8 +448,9 @@ def get_data(base_url, hts, site, measurement, from_date=None, to_date=None, agg
             if tag1 == 'Gap':
                 gap = gap + 1
                 continue
-            if gap % 2 == 1:
-                continue
+            if not ignore_gaps:
+                if gap % 2 == 1:
+                    continue
             tsdata_list.append([d.find('T').text, int(d.find(g_type['row']).text.encode('ascii', 'ignore').decode()) * g_type['multiplier']])
         data_df = pd.DataFrame(tsdata_list, columns=['DateTime', 'Value'])
 
@@ -411,13 +462,22 @@ def get_data(base_url, hts, site, measurement, from_date=None, to_date=None, agg
 
     ### If detection limit values should be estimated
     if isinstance(dtl_method, str):
-        less1 = data_df['Value'].str.contains('<')
-        if less1.sum() > 0:
-            less1.loc[less1.isnull()] = False
-            data_df = data_df.copy()
-            data_df.loc[less1, 'Value'] = pd.to_numeric(data_df.loc[less1, 'Value'].str.replace('<', ''), errors='coerce') * 0.5
-            data_df['Value'] = data_df['Value'].astype('float32')
-            if dtl_method == 'trend':
+        if data_df['Value'].dtype == 'object':
+            greater1 = data_df['Value'].str.contains('>')
+            if greater1.sum() > 0:
+                greater1.loc[greater1.isnull()] = False
+                data_df = data_df.copy()
+                data_df.loc[greater1, 'Value'] = pd.to_numeric(data_df.loc[greater1, 'Value'].str.replace('>', ''), errors='coerce') * 2
+            data_df['Value'] = pd.to_numeric(data_df['Value'], errors='ignore')
+
+        if data_df['Value'].dtype == 'object':
+            less1 = data_df['Value'].str.contains('<')
+            if less1.sum() > 0:
+                less1.loc[less1.isnull()] = False
+                data_df = data_df.copy()
+                data_df.loc[less1, 'Value'] = pd.to_numeric(data_df.loc[less1, 'Value'].str.replace('<', ''), errors='coerce') * 0.5
+            data_df['Value'] = pd.to_numeric(data_df['Value'], errors='coerce')
+            if (dtl_method == 'trend') and (less1.sum() > 0):
                 df1 = data_df.loc[less1]
                 count1 = len(data_df)
                 count_dtl = len(df1)
@@ -478,7 +538,7 @@ def wq_sample_parameter_list(base_url, hts, site):
     url = build_url(base_url, hts, 'GetData', site, 'WQ Sample')
 
     ### Request data and load in xml
-    resp = requests.get(url)
+    resp = requests.get(url, timeout=300)
     tree1 = ET.fromstring(resp.content)
     if tree1.find('Error') is not None:
         raise ValueError('No results returned from URL request')
@@ -487,7 +547,8 @@ def wq_sample_parameter_list(base_url, hts, site):
         print('No data, returning empty DataFrame')
         return pd.DataFrame()
     data1 = meas1.find('Data')
-    es1 = data1.getchildren()
+    # es1 = data1.getchildren()
+    es1 = list(data1)
 
     ### Extract data
     tsdata_dict = {}
